import { execa } from 'execa';
import editJsonFile from 'edit-json-file';
import path from 'path';
import { dirName } from './utils/path.js';
import Spinner from './spinner.js';
import { printGitInit, printYarnInstall } from './utils/console.js';
import copy from './utils/file.js';
import DEFAULT_TEMPLATE from './utils/string.js';

class PackageManager {
  #projectName;

  #package;

  #dependencies;

  #devDependencies;

  init(projectName) {
    const packagePath = path.join(dirName, projectName, '/package.json');
    const packageJson = editJsonFile(packagePath);
    this.#projectName = projectName;
    this.#package = packageJson;
    this.#dependencies = packageJson.get('dependencies');
    this.#devDependencies = packageJson.get('devDependencies');
    packageJson.set('name', projectName);
  }

  async install() {
    const spinner = Spinner;
    const gitInit = execa('git', ['init'], { cwd: this.#projectName });
    await copy(`${DEFAULT_TEMPLATE}/gitignore`, `${this.#projectName}/.gitignore`);
    await gitInit;
    printGitInit();
    printYarnInstall();
    const yarnInstall = execa('yarn', [], { cwd: this.#projectName });
    yarnInstall.stdout.on('data', spinner.installSpinner);
    await yarnInstall;
    spinner.finishSpinner();
    const yarnLint = execa('yarn', ['lint'], { cwd: this.#projectName });
    await yarnLint;
  }

  addModules(type) {
    if (type.includes('pinia')) this.#addPinia();
    if (type.includes('vue-router')) this.#addVueRouter();
    if (type.includes('vue-i18n')) this.#addI18n();
    if (type.includes('io-ts')) this.#addIoTs();
    if (type.includes('jest')) this.#addJest();
    if (type.includes('cypress')) this.#addCypress();
    if (type.includes('storybook')) this.#addStorybook();
    this.#save();
  }

  #save() {
    this.#package.set('dependencies', this.#dependencies);
    this.#package.set('devDependencies', this.#devDependencies);
    this.#package.save();
  }

  #addPinia() {
    this.#dependencies.pinia = '^2.0.17';
  }

  #addVueRouter() {
    this.#dependencies['vue-router'] = '^4.1.3';
  }

  #addI18n() {
    this.#dependencies['vue-i18n'] = '^9.2.0';
  }

  #addIoTs() {
    this.#dependencies['io-ts'] = '^2.2.17';
    this.#dependencies['fp-ts'] = '^2.12.2';
  }

  #addJest() {
    this.#package.set('scripts.jest', 'vue-cli-service test:unit --watchAll');
    this.#devDependencies['@vue/cli-plugin-unit-jest'] = '~5.0.0';
    this.#devDependencies['@vue/vue3-jest'] = '^28.0.1';
    this.#devDependencies['babel-jest'] = '^28.1.3';
    this.#devDependencies.jest = '^28.1.3';
  }

  #addCypress() {
    this.#package.set('scripts.cypress', "NODE_ENV='development' exec -- vue-cli-service test:e2e");
    this.#devDependencies.cypress = '^10.4.0';
    this.#devDependencies['@vue/cli-plugin-e2e-cypress'] = '~5.0.0';
  }

  #addStorybook() {
    this.#package.set('scripts.storybook', 'start-storybook -p 6006');
    this.#package.set('scripts.build-storybook', 'build-storybook');
    this.#devDependencies['@storybook/addon-actions'] = '^6.5.10';
    this.#devDependencies['@storybook/addon-essentials'] = '^6.5.10';
    this.#devDependencies['@storybook/addon-links'] = '^6.5.10';
    this.#devDependencies['@storybook/builder-webpack5'] = '^6.5.10';
    this.#devDependencies['@storybook/manager-webpack5'] = '^6.5.10';
    this.#devDependencies['@storybook/vue3'] = '^6.5.10';
  }
}

export default new PackageManager();
